unit rdbtool_ADO;

{<
  rdbtool_ADO is set functions for working with ADO datasets,
  ChangeSortExADO and RequeryExADO can be used by rDBGrid
  for sorting and refreshing of data if ADODataset is used.
}

interface

{$INCLUDE _rosi.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Db, ADODb, Variants, Forms, DBGrids, rDBGrid;

function SetTableIndexADO(T: TADOTable; FieldName: string): boolean;
procedure ChangeSortADO(ADO: TCustomADODataSet; SortFieldName: String; SortDesc: boolean; var Accept: boolean);
procedure ChangeSortExADO(ADO: TCustomADODataSet; SortFieldNames: array of string; SortDirections: array of boolean; var Accept: boolean);
function RequeryExADO(ADO: TCustomADODataSet; F: TField): boolean;
function RequeryGridADO(rDBGrid: TrDBGrid): boolean; overload;
function RequeryGridADO(rDBGrid: TrDBGrid; ToFirst: boolean): boolean; overload;

implementation

uses rstring, rtool, rdbtool, rdlg;

function SetTableIndexADO(T: TADOTable; FieldName: string): boolean;
begin
  Result:=false;
  try
    T.IndexFieldNames:=FieldName;
    Result:=true;
  except
  end;
end;

procedure ChangeSortADO(ADO: TCustomADODataSet; SortFieldName: String; SortDesc: boolean; var Accept: boolean);
const
  FieldSep = ',';
var
  B: TBookmark;
  F: TField;
  oldCursor: TCursor;
  i: integer;
  aFN, sFN: string;
begin
  oldCursor:=SQLCursor;
  try
    Accept:=ADO.Active and (not IsEditing(ADO));
    if not Accept then
      Abort;
    sFN:='';
    if SortFieldName <> '' then
    begin
      i:=0;
      repeat
        Inc(i);
        aFN:=FindStrPart(SortFieldName, FieldSep, i, false);
        if aFN = '' then
          Break;
        F:=ADO.FindField(aFN);
        Accept:=Assigned(F) and IsSimpleField(F);
        if not Accept then
          Abort;
        if (F.FieldKind = fkLookup) then
          aFN:=F.KeyFields;
        if Pos(' ', aFN) > 0 then
          aFN:='[' + aFN + ']';
        if SortDesc then
          aFN:=aFN + ' DESC';
        if sFN <> '' then
          sFN:=sFN + FieldSep;
        sFN:=sFN + aFN;
      until false;
    end;
    B:=ADO.GetBookmark;
    try
      ADO.Sort:=sFN;
      ADO.GotoBookmark(B);
    finally
      ADO.FreeBookmark(B);
    end;
  except
    Accept:=false;
  end;
  ChCursor(oldCursor);
end;

procedure ChangeSortExADO(ADO: TCustomADODataSet; SortFieldNames: array of string; SortDirections: array of boolean; var Accept: boolean);
const
  FieldSep = ',';
var
  B: TBookmark;
  F: TField;
  oldCursor: TCursor;
  i: integer;
  aFN, sFN: string;
begin
  // ArrOfStringToArrOfConst(SortFieldNames);
  oldCursor:=SQLCursor;
  try
    Accept:=ADO.Active and (not IsEditing(ADO));
    if not Accept then
      Abort;
    sFN:='';
    for i:=0 to High(SortFieldNames) do
    begin
      aFN:=SortFieldNames[i];
      F:=ADO.FindField(aFN);
      Accept:=Assigned(F) and IsSimpleField(F);
      if not Accept then
        Abort;
      if (F.FieldKind = fkLookup) then
        aFN:=F.KeyFields;
      if Pos(' ', aFN) > 0 then
        aFN:='[' + aFN + ']';
      if (i <= High(SortDirections)) and SortDirections[i] then
        aFN:=aFN + ' DESC';
      if sFN <> '' then
        sFN:=sFN + FieldSep;
      sFN:=sFN + aFN;
    end;
    B:=ADO.GetBookmark;
    try
      ADO.Sort:=sFN;
      ADO.GotoBookmark(B);
    finally
      ADO.FreeBookmark(B);
    end;
  except
    Accept:=false;
  end;
  ChCursor(oldCursor);
end;

function RequeryExADO(ADO: TCustomADODataSet; F: TField): boolean;
var
  B: TBookmark;
  oldCursor: TCursor;
  oldId: variant;
  oldSort: string;
  ToLast: boolean;
begin
  oldCursor:=SQLCursor;
  Result:=false;
  try
    if not ADO.Active then
      ADO.Open
    else
    begin
      // Find record after requery by old field value
      if Assigned(F) then
      begin
        oldId:=F.AsVariant;
        ADO.Requery();
        Result:=ADO.Locate(F.FieldName, oldId, []);
        if not Result then
          ADO.First;
      end
      else
      begin
        // Find record after requery by Bookmark
        B:=ADO.GetBookmark;
        ToLast:=false;
        try
          ADO.Requery();
          try
            ADO.GotoBookmark(B);
            Result:=true;
          except
            ToLast:=true;
          end;
        finally
          ADO.FreeBookmark(B);
        end;
        if ToLast then
        begin
          oldSort:=ADO.Sort;
          ADO.Sort:='';
          ADO.Last;
          B:=ADO.GetBookmark;
          try
            ADO.Sort:=oldSort;
            ADO.GotoBookmark(B);
          finally
            ADO.FreeBookmark(B);
          end;
        end;
      end;
    end;
  finally
    ChCursor(oldCursor);
  end;
end;

function RequeryGridADO(rDBGrid: TrDBGrid): boolean;
begin
  Result:=RequeryGridADO(rDBGrid, true);
end;

function RequeryGridADO(rDBGrid: TrDBGrid; ToFirst: boolean): boolean;
begin
  Result:=false;
  if rDBGrid.DataSource.DataSet is TCustomADODataSet then
    Result:=RequeryExADO(rDBGrid.DataSource.DataSet as TCustomADODataSet, nil);
  rDBGrid.ReSort;
  if ToFirst then
    rDBGrid.DataSource.DataSet.First;
end;

end.
